@@ -1,6 +1,7 @@
 ;;; postgres-qcons.scm --- construct SELECT queries
 
-;; Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011 Thien-Thi Nguyen
+;; Copyright (C) 2005, 2006, 2007, 2008, 2009,
+;;   2010, 2011, 2012 Thien-Thi Nguyen
 ;;
 ;; This file is part of Guile-PG.
 ;;
@@ -48,6 +49,7 @@
             sql-unpre
             sql-quote-auto-E?
             sql-quote
+            string-xrep
             idquote
             make-comma-separated-tree
             make-WHERE-tree
@@ -60,7 +62,17 @@
             parse+make-SELECT/tail-tree
             parse+make-SELECT-tree
             sql<-trees
-            sql-command<-trees))
+            sql-command<-trees)
+  #:use-module ((srfi srfi-13) #:select (string-index
+                                         string-prefix?
+                                         string-concatenate-reverse
+                                         substring/shared))
+  #:use-module ((srfi srfi-14) #:select (char-set:full
+                                         char-set-filter
+                                         char-set-size
+                                         char-set-union
+                                         char-set-for-each
+                                         ucs-range->char-set)))
 
 
 ;;; mirroring bootstrap
@@ -204,11 +216,11 @@
 ;; @result{} #f
 ;;
 ;; (e.g.)
-;; @print{} 'a\b'
+;; @print{} 'a\134b'
 ;;
 ;; (with-fluids ((sql-quote-auto-E? #t))
 ;;   (e.g.))
-;; @print{} E'a\b'
+;; @print{} E'a\134b'
 ;; @end example
 ;;
 ;; If there are no backslash characters, this has no effect.
@@ -217,16 +229,22 @@
 ;;
 (define sql-quote-auto-E? (make-fluid))
 
-;; Return a new string made by doubling each single-quote in string
-;; @var{s}, and prefixing and suffixing with single-quote.
+;; Return a new string made by doubling each single-quote in
+;; @var{string}, and prefixing and suffixing with single-quote.
+;; The returned string will also have backslash characters @code{#\\}
+;; replaced with @code{\134} (i.e., the string of four characters:
+;; @code{#\\}, @code{#\1}, @code{#\3}, @code{#\4}).
 ;; The returned string is marked by @code{sql-pre}.  For example:
 ;;
 ;; @lisp
-;; (define bef "ab'cd")
+;; (define bef "ab'c\\d")
 ;; (define aft (sql-quote bef))
-;; aft @result{} "'ab''cd'"
-;; (map string-length (list bef aft)) @result{} (5 8)
-;; (map sql-pre? (list bef aft)) @result{} (#f #t)
+;; (define (both proc)
+;;   (map proc (list bef aft)))
+;;
+;; (display aft)        @print{} 'ab''c\134d'
+;; (both string-length) @result{} (6 12)
+;; (both sql-pre?)      @result{} (#f #t)
 ;; @end lisp
 ;;
 ;; Note that this procedure used to return internal single-quote
@@ -236,9 +254,20 @@
 ;; is standards-conforming, upward compatible, and avoids futzing with
 ;; the runtime parameters.
 ;;
-(define (sql-quote s)
-  (or (string? s) (error "not a string:" s))
-  (let* ((olen (string-length s))
+(define (sql-quote string)
+  (or (string? string) (error "not a string:" string))
+  ;; Normalize before ‘E’ processing to handle introduced ‘#\\’ chars.
+  (let* ((s (let loop ((acc '()) (start 0))
+              (cond ((string-index string #\\ start)
+                     => (lambda (idx)
+                          (loop (cons* "\\134"
+                                       (substring/shared string start idx)
+                                       acc)
+                                (1+ idx))))
+                    ((null? acc) string)
+                    (else (string-concatenate-reverse
+                           acc (substring/shared string start))))))
+         (olen (string-length s))
          (E? (and (fluid-ref sql-quote-auto-E?)
                   (string-index s #\\)))
          (left-pos (if E? 1 0))
@@ -271,6 +300,84 @@
 (define (fs s . args)
   (apply simple-format #f s args))
 
+;; {string external representation}
+;;
+;; Guile 1.8 and later represent certain octets in the @dfn{xrep}, or
+;; external representation, of a string with @code{\xXX}, e.g.,
+;; @code{\x07} for @code{#\bel}.
+;; PostgreSQL 7.4 and 8.0 do not recognize @code{\xXX} as valid syntax
+;; in a literal string, which is a problem if you need to connect to
+;; them.
+;;
+;; But even for PostgreSQL 8.1 and later, which accepts @code{\xXX} as
+;; an octet with hex value @code{XX}, a problem arises when there is a
+;; mix of contiguous raw and @code{\x} representations, intended to
+;; represent (for instance) a UTF-8 encoded character.
+;; It seems@footnote{
+;; The survey and its results are in the @samp{guile-user} archive:@*
+;; @url{http://lists.gnu.org/archive/html/guile-user/2012-01/msg00114.html}.}
+;; Guile
+;;
+;; @format
+;;   - 1.4 DTRT by doing nothing (mu power!);
+;;   - 1.6 likewise is blissfully unperturbing;
+;;   - 1.8 fails by @code{\x}-escaping inconsistently;
+;;   - 2.0 doesn't have this problem.
+;; @end format
+;;
+;; @noindent
+;; To avoid both problems, use @code{string-xrep}.
+
+;; Return the external representation of @var{string},
+;; guaranteed to not have @code{\x}-escape sequences.
+;; Specifically, the first and last characters are double-quote @code{#\"},
+;; and internal @code{#\\} and @code{#\"} are backslash-escaped.
+;; Characters that normally have an xrep with @code{\x} are instead
+;; passed straight through.  For some Guile versions, this behavior
+;; is identical to @code{object->string}.
+;;
+;;-category: procedure
+;;-args: (1 0 0 string)
+;;
+(define string-xrep
+  (or (let ((dq (string #\"))
+            (ugh (char-set-filter
+                  (lambda (ch)
+                    (string-prefix? "\"\\x" (object->string (string ch))))
+                  char-set:full)))
+        (and (positive? (char-set-size ugh))
+             ;; Lame.
+             (let ((v (make-vector 256)))
+               (define (escape ch)
+                 (vector-ref v (char->integer ch)))
+               (set! ugh (ucs-range->char-set #o177 #o400 #t
+                                              (char-set-union
+                                               (char-set #\\ #\")
+                                               ugh)))
+               (char-set-for-each
+                (lambda (ch)
+                  (vector-set! v (char->integer ch)
+                               (case ch
+                                 ((#\\) "\\\\")
+                                 ((#\") "\\\"")
+                                 (else (string ch)))))
+                ugh)
+               ;; string-xrep
+               (lambda (s)
+                 (let loop ((acc (list dq)) (start 0))
+                   (cond ((string-index s ugh start)
+                          => (lambda (idx)
+                               (loop (cons* (escape (string-ref s idx))
+                                            (substring/shared s start idx)
+                                            acc)
+                                     (1+ idx))))
+                         ((zero? start) (object->string s))
+                         (else (string-concatenate-reverse
+                                (cons (substring/shared s start) acc)
+                                dq))))))))
+      ;; Cool.
+      object->string))
+
 ;; Return the @dfn{quoted identifier} form of @var{id}, a string
 ;; or symbol.  The returned string is marked by @code{sql-pre}.
 ;; For example:
@@ -308,29 +415,34 @@
                  id))
           (ra (string-index s #\[))
           (dot (string-index s #\.)))
+     (define (from beg)
+       (substring/shared s beg))
+     (define (xsub beg end)
+       (string-xrep (substring/shared s beg end)))
      (cond
       ;; Fast path; no complications.
       ((not (or ra dot))
-       (object->string s))
+       (string-xrep s))
       ;; Just dot (ab.cd => "ab"."cd", but ab.* => "ab".*).
       ((and dot (not ra))
-       (fs "~S.~S"
-           (substring s 0 dot)
-           (let ((after (substring s (1+ dot))))
-             (if (string=? "*" after)
-                 '*
-                 after))))
+       (string-append (xsub 0 dot)
+                      "."
+                      (let ((after (1+ dot)))
+                        ((if (and (= (1- (string-length s)) after)
+                                  (char=? #\* (string-ref s after)))
+                             identity
+                             string-xrep)
+                         (from after)))))
       ;; Just array (abcd[xyz] => "abcd"[xyz]).
       ((and ra (not dot))
-       (fs "~S~A"
-           (substring s 0 ra)
-           (substring s ra)))
+       (string-append (xsub 0 ra)
+                      (from ra)))
       ;; Both dot and array (ab.cd[xyz] => "ab"."cd"[xyz]).
       (#t
-       (fs "~S.~S~A"
-           (substring s 0 dot)
-           (substring s (1+ dot) ra)
-           (substring s (ra))))))))
+       (string-append (xsub 0 dot)
+                      "."
+                      (xsub (1+ dot) ra)
+                      (from ra)))))))
 
 (define (maybe-dq sym)
   ;; Hmmm, why not use ‘idquote’ also for this?
@@ -512,7 +624,7 @@
                     ((symbol? x) (maybe-dq x))
                     ((and (pair? x) (string? (car x)))
                      (as (expr (cdr x))
-                         (sql-pre (fs "~S" (car x)))))
+                         (sql-pre (string-xrep (car x)))))
                     ((pair? x) (expr x))
                     (else (error "bad col spec:" x))))
             cols))
